/*
 * lab3.c
 *
 * Created: 3/16/2017 10:48:08 PM
 * Class  : Masters-2017 (AD3)
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>

// Include the header file for sleep function
#include <avr/sleep.h>
#include <util/delay.h>


//USART baud rate calculation
#define F_CPU 3333333
#define CLK_PER (F_CPU)
#define USART0_BAUD_RATE(F_BAUD) ((float)CLK_PER * 64 / (16 * (float)F_BAUD))

// Variables for temperature sensor calibration

uint32_t temperature_deg=0;
int8_t sigrow_offset = 0;	// Read signed value from signature row
uint8_t sigrow_gain = 0;	// Read unsigned value from signature row
volatile char window_hit_flag=0;


void usart_init()
{
	PORTB.DIRSET|= (1<<PIN2_bp);//PB2 as output
	PORTB.OUTCLR|= (1<<PIN2_bp);// initial level false
	
	// USART baud rate initialization as mentioned in the datasheet
	
	int8_t sigrow_value  = SIGROW.OSC20ERR3V;				// Read signed error
	int32_t baud          = 115200;							// Ideal baud rate

	baud *= (1024 + sigrow_value);							// Sum resolution + Error
	baud /=  1024;											// Divide by resolution

	USART0.BAUD=(uint16_t)USART0_BAUD_RATE(baud);			// Set adjusted baud rate
	USART0.CTRLB=(1<<USART_TXEN_bp)|(1<<USART_SFDEN_bp);	// Transmitter enable (normal mode)
}


void USART_Transmit( char* data )
{
	while(*data)
	{
		// Wait for USART Data Register Empty Bit
		while ( !( USART0.STATUS & (1<<USART_DREIF_bp)) );
		
		// Write the byte to the USART DATA Register  
		USART0.TXDATAL = *data++;
	}

	while ( !( USART0.STATUS & (1<<USART_TXCIF_bp)) );
	USART0.STATUS |= USART_TXCIF_bm;
}


void USART_temperature(const uint8_t data)
{
	while ( !( USART0.STATUS & (1<<USART_DREIF_bp)) );	   // ASCII conversion

	USART0.TXDATAL = ((data/10)+48);
	while ( !( USART0.STATUS & (1<<USART_DREIF_bp)) );

	USART0.TXDATAL=((data%10)+48);
	while ( !( USART0.STATUS & (1<<USART_DREIF_bp)) );

	USART0.TXDATAL = ('C');
	while ( !( USART0.STATUS & (1<<USART_TXCIF_bp)) );

	USART0.STATUS |= USART_TXCIF_bm;
}


uint32_t convert_temperature_deg(uint32_t temperature)
{
	sigrow_offset= SIGROW.TEMPSENSE1;
	sigrow_gain = SIGROW.TEMPSENSE0;
	temperature = temperature - sigrow_offset;
	temperature *= sigrow_gain;
	temperature += 0x80;
	temperature >>= 8;
	temperature -=273;

	return temperature;
}


void adc_init()
{
	// ADC Vref voltage selection
    VREF.CTRLA = VREF_ADC0REFSEL_1V1_gc; 												//1.1V Internal reference 
    
	// Prescaler setting and sample capacitance selection
	ADC0.CTRLC = ADC_PRESC_DIV4_gc |(1<<ADC_SAMPCAP_bp); 								//Clock presclar clck_per/4, Select sample capacitance
	
	// Set the initial delay and sampling time(greater than 32us x Fclk_adc)
	ADC0.CTRLD = (1<<ADC_INITDLY0_bp); 													//Initial delay
	ADC0.SAMPCTRL = (1<<ADC_SAMPLEN3_bp) | (1<<ADC_SAMPLEN2_bp)|(1<<ADC_SAMPLEN1_bp);   //Sample length

	// Select Temperature Sensor
	ADC0.MUXPOS = ADC_MUXPOS_TEMPSENSE_gc;
		
	// Set window threshold values for the application 
	ADC0.WINLT= 0x0150;
	ADC0.WINHT= 0x0250;
	
	// Select the window comparator mode as inside
	ADC0.CTRLE= ADC_WINCM_INSIDE_gc; 
	
	// Enable ADC window comparator interrupt
	ADC0.INTCTRL=(1<<ADC_WCMP_bp);
	
	// Set Run-in-Standby bit for ADC
	ADC0.CTRLA=(1<< ADC_ENABLE_bp)|(1<<ADC_RUNSTBY_bp); 
}


ISR(ADC0_WCOMP_vect)
{
	// Set the window_hit_flag to '1'
	window_hit_flag= 1;
	
	// Clear the interrupt flag
	ADC0.INTFLAGS= (1<<ADC_WCMP_bp); 
}


void rtc_init()
{
	// Select the clock source
	RTC.CLKSEL = RTC_CLKSEL_INT1K_gc;
	
	// Enter the period value as per calculation 
	RTC.PER = 100;
	
	//Set Run-in-Standby bit for RTC
	RTC.CTRLA = (1<<RTC_RTCEN_bp) | (1<<RTC_RUNSTDBY_bp);
}


// RTC ISR routine is not necessary
#if 0
ISR(RTC_CNT_vect)
{
	/* ... */
}
#endif


void sleep_init()
{
	// TO DO-20 : Change Idle sleep mode to Standby sleep mode
	set_sleep_mode(SLEEP_MODE_STANDBY);

	// Enable Sleep
	sleep_enable();
}


void event_init()
{
	// Select the event generator 
	EVSYS.ASYNCCH0 = EVSYS_ASYNCCH0_RTC_OVF_gc;
	
	// Select the event user
	EVSYS.ASYNCUSER1 = EVSYS_ASYNCUSER1_ASYNCCH0_gc;
	
	// Enable event input of ADC
	ADC0.EVCTRL=(1<<ADC_STARTEI_bp);
}


int main(void)
{
	// USART display message
	char message[] = {"\n\r The temperature is "}; 

	//Initialization of peripherals
	usart_init();
	rtc_init();
 	adc_init();
	
	// Initialize Event System
	 event_init();

	// Enable Global interrupt using intrinsic function
	sei();
	
	// Initialize the sleep
	sleep_init(); 
	
	while (1)
	{
		// Check for ADC window hit flag
		
		if(window_hit_flag)
		{
			// Reading temperature 
			temperature_deg=ADC0.RES;
			
			// Calling Temperature conversion function 
			temperature_deg=convert_temperature_deg(temperature_deg); 
			
			// Transmit the message via USART
			USART_Transmit(message);
			USART_temperature(temperature_deg);
			
			// Clear the ADC window hit flag
			window_hit_flag=0;
		}

		// Make the CPU to enter in to Sleep mode
		sleep_cpu();
	}
}

